#  Copyright (c) Meta Platforms, Inc. and affiliates.

import logging

logger = logging.getLogger("tritonparse")
# Configure logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
